#ifndef _VM_H
#define _VM_H

#include "pgm.h"		/* struct PGMDEF */
#include "list.h"		/* struct node */

/* ------------------------------------------------------------------------- */
typedef int Int;
typedef double Real;

typedef enum
{
  op_illegalInstr,
  op_loadi,
  op_loadf,
  op_loads,
  op_undef,
  op_emptyl,
  op_loadLoc,
  op_loadAbs,
  op_storeLoc,
  op_storeAbs,
  op_pop,
  op_dup,
  op_pushj,
  op_popj,
  op_entry,
  op_exit,
  op_brTrue,
  op_brFalse,
  op_jump,

  op_svc_exit,
  op_svc_getArgs,
  op_svc_load,
  op_svc_store,
  op_svc_write,
  op_svc_writeln,

  op_addi,
  op_subi,
  op_muli,
  op_divi,
  op_modi,
  op_maxi,
  op_mini,
  op_eqi,
  op_gei,
  op_gti,

  op_addf,
  op_subf,
  op_mulf,
  op_divf,
  op_maxf,
  op_minf,
  op_eqf,
  op_gef,
  op_gtf,

  op_i2s,
  op_f2s,
  op_i2f,
  op_trunc,
  op_round,
  op_concs,

  op_isemptyl,
  op_lengthl,
  op_taill,
  op_concl,
  op_consl,
  op_appendl,
  op_indexl,

  op_width,
  op_height,
  op_black,
  op_white,
  op_grey,
  op_gamma,
  op_invert,
  op_bitmap,
  op_blackAndWhite,
  op_reduceColor,
  op_flipHorizontal,
  op_flipVertical,
  op_flipDiagonal,
  op_rotate,
  op_shift,
  op_cut,
  op_paste,
  op_scale,
  op_replicate,
  op_resize,
  op_sideBySide,
  op_above,
  op_partitionHorizontal,
  op_partitionVertical,
  op_splitHorizontal,
  op_splitVertical,
  op_mergeHorizontal,
  op_mergeVertical,
  op_concatHorizontal,
  op_concatVertical,
  op_mean,
  op_diff,
  op_inverseMean,
  op_inverseDiff,
  op_mulp,
  op_maxp,
  op_minp
    /* add new image processing operations here... */
}
Opcode;

/* machine instruction: opcode [operand] */
typedef struct
{
  Opcode opcode;
  void *operand;
}
Instr;

extern int pplvm (int argc,
		  char *argv[],
		  Instr * instructions,
		  int codeSegmentLength,
		  int dataSegmentLength);

/* ------------------------------------------------------------------------- */

/* built-in machine values */
enum types
{
  INT,
  DOUBLE,
  STRING,
  PICTURE,
  LIST,
  ADDRESS,
  VOID				/* undef */
};

union machineValue
{
  int integer;
  double real;
  int address;
  char *string;
  struct PGMDEF picture;
  struct node *list;
};

/* operand have a reference count */
struct value
{
  int refCount;
  union machineValue value;
};

/* operands are tagged */
struct operand
{
  enum types tag;
  struct value *value;		/* all operands go on the heap */
};


/* machine status */
struct machineStatus
{
  int ip;			/* instruction pointer  */
  Instr instr;			/* current instruction  */
  int sp;			/* stack pointer        */
  int running;			/* flag...              */
};

/* global variables: */
extern struct machineStatus sr;	/* status register */
extern int memSize;		/* size of global data segment */
extern int codeSize;		/* number of instructions, i. e. size of array */
extern struct operand *m;	/* global memory */
extern struct operand *stack;	/* the evaluation stack */
extern int stackSize;

/* "stack" frames */
struct frame
{
  int size;
  struct operand *l;		/* local mem */
  struct frame *prev;
};

extern struct frame *frame;	/* list of frames */

/* list of command line arguments */
extern int argCount;
extern char **args;


#ifdef DEBUG
#define ERR(x, y) fprintf(stderr, (x), (y)); (void *) fflush(stderr)
#else
#define ERR(x, y) ;
#endif

#define OUT(x, y) fprintf(stdout, (x), (y)); (void *) fflush(stdout)

#endif
